@@ -1,23 +1,27 @@
-from __future__ import division
-
+from __future__ import absolute_import, division, print_function
 import multiprocessing
 import numexpr
 import numpy as np
+import operator
 try:
     import pyfftw
 except:
     pass
+import scipy.sparse as sp
+import sys
 from itertools import izip
 from scipy.sparse.linalg import eigsh
 from .core import (
-    Operator, BlockRowOperator, BroadcastingBase, CompositionOperator,
-    ConstantOperator, DiagonalOperator, HomothetyOperator, ReductionOperator,
-    _pool)
-from .flags import inplace, linear, real, square, symmetric
+    BlockRowOperator, BroadcastingBase, CompositionOperator, ConstantOperator,
+    DiagonalBase, DiagonalOperator, HomothetyOperator, IdentityOperator,
+    Operator, ReductionOperator, ZeroOperator, operation_assignment, _pool)
+from .flags import (
+    contiguous, idempotent, inplace, linear, real, square, symmetric,
+    update_output)
 from .memory import empty
 from .utils import (
-    broadcast_shapes, cast, complex_dtype, float_dtype, isalias,
-    izip_broadcast, pi, product, strshape, tointtuple)
+    broadcast_shapes, cast, complex_dtype, float_dtype, inspect_special_values,
+    isalias, izip_broadcast, pi, product, strshape, tointtuple, ufuncs)
 from .warnings import warn, PyOperatorsWarning
 
 __all__ = [
@@ -29,10 +33,12 @@ __all__ = [
     'DifferenceOperator',
     'EigendecompositionOperator',
     'IntegrationTrapezeOperator',
+    'MaskOperator',
     'PackOperator',
     'RadiansOperator',
     'Rotation2dOperator',
     'Rotation3dOperator',
+    'SparseOperator',
     'SumOperator',
     'SymmetricBandOperator',
     'SymmetricBandToeplitzOperator',
@@ -182,17 +188,17 @@ class DenseBlockDiagonalOperator(DenseBase):
     >>> np.shape(data)
     (2, 1, 3)
     >>> d = DenseBlockDiagonalOperator(data, dtype=int)
-    >>> print(d(np.ones(3))).shape  # the input is broadcast
+    >>> print(d(np.ones(3)).shape)  # the input is broadcast
     (2, 1)
     >>> print(d.todense(shapein=3))
     [[ 1  1  1]
      [ 1 -1  1]]
-    >>> print(d(np.ones([2, 3]))).shape
+    >>> print(d(np.ones([2, 3])).shape)
     (2, 1)
     >>> print(d.todense(shapein=(2, 3)))
     [[ 1  1  1  0  0  0]
      [ 0  0  0  1 -1  1]]
-    >>> print(d(np.ones([3, 2, 3]))).shape
+    >>> print(d(np.ones([3, 2, 3])).shape)
     (3, 2, 1)
     >>> print(d.todense(shapein=(3, 2, 3)))
     [[ 1  1  1  0  0  0  0  0  0  0  0  0  0  0  0  0  0  0]
@@ -283,12 +289,12 @@ class DenseOperator(DenseBlockDiagonalOperator):
     ...      [np.sin(theta),  np.cos(theta)]]
     >>> input = [[1, 0], [0, 1], [-1, 0], [0, -1]]
     >>> op = DenseOperator(m)
-    >>> print op(input)
+    >>> print(op(input))
     [[ 0.70710678  0.70710678]
      [-0.70710678  0.70710678]
      [-0.70710678 -0.70710678]
      [ 0.70710678 -0.70710678]]
-    >>> print op.T(op(input))
+    >>> print(op.T(op(input)))
     [[ 1.  0.]
      [ 0.  1.]
      [-1.  0.]
@@ -314,7 +320,134 @@ class DenseOperator(DenseBlockDiagonalOperator):
         return shape[:-self.naxesout] + self._sn
 
 
-class DiagonalNumexprOperator(DiagonalOperator):
+@linear
+@contiguous
+@update_output
+class SparseBase(Operator):
+    def __init__(self, matrix, dtype=None, shapein=None, shapeout=None,
+                 **keywords):
+        if dtype is None:
+            dtype = matrix.dtype
+        if shapein is None:
+            shapein = matrix.shape[1]
+        elif product(shapein) != matrix.shape[1]:
+            raise ValueError(
+                "The input shape '{0}' is incompatible with the sparse matrix "
+                "shape {1}.".format(shapein, matrix.shape))
+        if shapeout is None:
+            shapeout = matrix.shape[0]
+        elif product(shapeout) != matrix.shape[0]:
+            raise ValueError(
+                "The output shape '{0}' is incompatible with the sparse matrix"
+                " shape {1}.".format(shapeout, matrix.shape))
+        if not hasattr(matrix, 'nbytes'):
+            self._set_nbytes(matrix)
+        self.matrix = matrix
+        Operator.__init__(self, shapein=shapein, shapeout=shapeout,
+                          dtype=dtype, **keywords)
+
+    @property
+    def nbytes(self):
+        return self.matrix.nbytes
+
+    @staticmethod
+    def _set_nbytes(m):
+        if isinstance(m, (sp.csc_matrix, sp.csr_matrix, sp.bsr_matrix)):
+            m.nbytes = m.data.nbytes + m.indices.nbytes + m.indptr.nbytes
+        elif isinstance(m, sp.coo_matrix):
+            m.nbytes = m.data.nbytes + 2 * m.row.nbytes
+        elif isinstance(m, sp.dia_matrix):
+            m.nbytes = m.data.nbytes + m.offsets.nbytes
+        elif isinstance(m, sp.dok_matrix):
+            sizeoftuple = sys.getsizeof(())
+            m.nbytes = (24 * m.ndim + m.dtype.itemsize +
+                        2 * sizeoftuple + 24) * len(m.viewitems())
+        else:
+            raise TypeError("The sparse format '{0}' is not handled."
+                            .format(type(m)))
+
+
+@linear
+@contiguous
+class SparseOperator(SparseBase):
+    """
+    Operator handling sparse matrix storages.
+
+    The sparse storage can be anyone from the scipy.sparse package (except
+    the LIL format, which is not suited for matrix-vector multiplication):
+        - bsr_matrix: Block Sparse Row matrix
+        - coo_matrix: A sparse matrix in COOrdinate format
+        - csc_matrix: Compressed Sparse Column matrix
+        - csr_matrix: Compressed Sparse Row matrix
+        - dia_matrix: Sparse matrix with DIAgonal storage
+        - dok_matrix: Dictionary Of Keys based sparse matrix
+
+    Example
+    -------
+    >>> from scipy.sparse import csr_matrix
+    >>> sm = csr_matrix([[1, 0, 2, 0],
+    ...                  [0, 0, 3, 0],
+    ...                  [4, 5, 6, 0],
+    ...                  [1, 0, 0, 1]])
+    >>> so = SparseOperator(sm)
+    >>> so([1, 0, 0, 0])
+    array([1, 0, 4, 1])
+    >>> so.T([1, 0, 0, 0])
+    array([1, 0, 2, 0])
+
+    """
+    def __init__(self, matrix, dtype=None, shapein=None, shapeout=None,
+                 **keywords):
+        """
+        matrix : sparse matrix from scipy.sparse
+           The sparse matrix to be wrapped into an Operator.
+
+        """
+        if not sp.issparse(matrix):
+            raise TypeError('The input sparse matrix type is not recognised.')
+        if isinstance(matrix, sp.lil_matrix):
+            raise TypeError('The LIL format is not suited for arithmetic opera'
+                            'tions.')
+        SparseBase.__init__(self, matrix, dtype=dtype, shapein=shapein,
+                            shapeout=shapeout, **keywords)
+        self.set_rule('T', lambda s: SparseOperator(s.matrix.transpose()))
+        self.set_rule(('.', HomothetyOperator), lambda s, o:
+                      SparseOperator(o * s.matrix), CompositionOperator)
+
+    def direct(self, input, output, operation=operation_assignment):
+        input = input.ravel().astype(output.dtype)
+        output = output.ravel()
+        if operation is operation_assignment:
+            output[...] = 0
+        elif operation is not operator.iadd:
+            raise ValueError('Invalid reduction operation.')
+        m = self.matrix
+        if isinstance(m, sp.dok_matrix):
+            for (i, j), v in m.iteritems():
+                output[i] += v * input[j]
+            return
+        M, N = m.shape
+        fn = getattr(sp.sparsetools, m.format + '_matvec')
+        if isinstance(m, (sp.csr_matrix, sp.csc_matrix)):
+            fn(M, N, m.indptr, m.indices, m.data, input, output)
+        elif isinstance(m, sp.coo_matrix):
+            fn(m.nnz, m.row, m.col, m.data, input, output)
+        elif isinstance(m, sp.bsr_matrix):
+            R, C = m.blocksize
+            fn(M // R, N // C, R, C, m.indptr, m.indices, m.data.ravel(),
+               input, output)
+        elif isinstance(m, sp.dia_matrix):
+            fn(M, N, len(m.offsets), m.data.shape[1], m.offsets, m.data,
+               input, output)
+        else:
+            raise NotImplementedError()
+
+    def todense(self, shapein=None, shapeout=None, inplace=False):
+        return self.matrix.toarray()
+
+
+@inplace
+class DiagonalNumexprOperator(DiagonalBase):
     """
     DiagonalOperator whose diagonal elements are calculated on the fly using
     the numexpr package and that can be separated when added or multiplied
@@ -355,8 +488,7 @@ class DiagonalNumexprOperator(DiagonalOperator):
         self._global_dict[var] = data.T \
             if broadcast is not None and broadcast.lower() == 'rightward' \
             else data
-        BroadcastingBase.__init__(self, data, broadcast, dtype=dtype,
-                                  **keywords)
+        DiagonalBase.__init__(self, data, broadcast, dtype=dtype, **keywords)
 
     def direct(self, input, output):
         if self.broadcast == 'rightward':
@@ -432,6 +564,55 @@ class IntegrationTrapezeOperator(BlockRowOperator):
         BlockRowOperator.__init__(self, ops, new_axisin=new_axisin, **keywords)
 
 
+@real
+@idempotent
+@inplace
+class MaskOperator(DiagonalBase):
+    """
+    A subclass of DiagonalOperator with 0 (True) and 1 (False) on the diagonal.
+
+    Exemple
+    -------
+    >>> M = MaskOperator([True, False])
+    >>> M.todense()
+    array([[0, 0],
+           [0, 1]])
+
+    Notes
+    -----
+    We follow the convention of MaskedArray, where True means masked.
+
+    """
+    def __init__(self, data, broadcast=None, **keywords):
+        data = np.array(data, dtype=bool, copy=False)
+        if broadcast is None:
+            broadcast = 'scalar' if data.ndim == 0 else 'disabled'
+        if broadcast == 'disabled':
+            keywords['shapein'] = data.shape
+        nmones, nzeros, nones, other, same = inspect_special_values(data)
+        if data.size in (nzeros, nones):
+            if nzeros == data.size:
+                self.__class__ = IdentityOperator
+                self.__init__(**keywords)
+                return
+            if nones == data.size:
+                keywords['flags'] = Operator.validate_flags(
+                    keywords.get('flags', {}), square=True)
+                self.__class__ = ZeroOperator
+                self.__init__(**keywords)
+                return
+        DiagonalBase.__init__(self, data, broadcast, **keywords)
+
+    def direct(self, input, output):
+        if self.broadcast == 'rightward':
+            ufuncs.masking(input.T, self.data.T, output.T)
+        else:
+            ufuncs.masking(input, self.data, output)
+
+    def get_data(self):
+        return ~self.data
+
+
 @real
 @linear
 class PackBase(BroadcastingBase):
@@ -449,10 +630,6 @@ class PackBase(BroadcastingBase):
             return (self.n,) + shape[self.data.ndim:]
         return shape[:-self.data.ndim] + (self.n,)
 
-    @staticmethod
-    def _rule_broadcast(b1, b2, operation):
-        return
-
     def _validate_packed(self, shape):
         actual = shape[0 if self.broadcast == 'rightward' else -1]
         if actual != self.n:
@@ -580,7 +757,7 @@ class Rotation2dOperator(DenseBlockDiagonalOperator):
     -------
     >>> r = Rotation2dOperator([45, 90], degrees=True)
     >>> r([1, 0])
-    >>> print r([1, 0])
+    >>> print(r([1, 0]))
     [[  7.07106781e-01   7.07106781e-01]
      [  6.12323400e-17   1.00000000e+00]]
 
@@ -642,13 +819,13 @@ class Rotation3dOperator(DenseBlockDiagonalOperator):
     Example
     -------
     >>> r1 = Rotation3dOperator("Y", 90, degrees=True)
-    >>> print r1([1, 0, 0])
+    >>> print(r1([1, 0, 0]))
     [  6.12323400e-17   0.00000000e+00  -1.00000000e+00]
     >>> r2 = Rotation3dOperator("XYZ", 30, 40, 50, degrees=True)
-    >>> print r2([1, 0, 0])
+    >>> print(r2([1, 0, 0]))
     [ 0.49240388  0.58682409 -0.64278761]d
     >>> r3 = Rotation3dOperator("ZY'X''", 50, 40, 30, degrees=True)
-    >>> print r3([1, 0, 0])
+    >>> print(r3([1, 0, 0]))
     [ 0.49240388  0.58682409 -0.64278761]
 
     """
@@ -1199,7 +1376,7 @@ class SymmetricBandToeplitzOperator(Operator):
     Example
     -------
     >>> N = SymmetricBandToeplitzOperator(5, [3, 2, 1, 1])
-    >>> print N.todense().astype(int)
+    >>> print(N.todense().astype(int))
     [[3 2 1 1 0]
      [2 3 2 1 1]
      [1 2 3 2 1]
@@ -1251,9 +1428,8 @@ class SymmetricBandToeplitzOperator(Operator):
     def direct(self, x, out):
         with _pool.get(self.fftsize, self.dtype, aligned=True,
                        contiguous=True) as rbuffer:
-            with _pool.get(
-                    self.fftsize // 2 + 1, complex_dtype(self.dtype),
-                    aligned=True, contiguous=True) as cbuffer:
+            with _pool.get(self.fftsize // 2 + 1, complex_dtype(self.dtype),
+                           aligned=True, contiguous=True) as cbuffer:
                 lpad = (self.bandwidth - 1) // 2
                 x = x.reshape((-1, self.nsamples))
                 out = out.reshape((-1, self.nsamples))
@@ -1330,8 +1506,8 @@ class EigendecompositionOperator(CompositionOperator):
     Inputs
     -------
 
-    A: LinearOperator (default: None)
-      The LinearOperator to approximate.
+    A: Operator (default: None)
+      The linear operator to approximate.
     v: 2d ndarray (default: None)
       The eigenvectors as given by arpack.eigsh
     w: 1d ndarray (default: None)
