@@ -1,3 +1,4 @@
+from __future__ import division, print_function
 import itertools
 import numpy as np
 import operator
@@ -12,9 +13,9 @@ from pyoperators import (
     Operator, AdditionOperator, BlockColumnOperator, BlockDiagonalOperator,
     BlockRowOperator, BlockSliceOperator, CompositionOperator,  GroupOperator,
     ConstantOperator, DenseOperator, DiagonalOperator, HomothetyOperator,
-    IdentityOperator, MaskOperator, MultiplicationOperator, PowerOperator,
+    IdentityOperator, MultiplicationOperator, PowerOperator,
     ReciprocalOperator, ReductionOperator, SparseOperator, SquareOperator,
-    ZeroOperator, asoperator, I, O, X)
+    asoperator, I, X)
 from pyoperators.core import CopyOperator, _pool as pool
 from pyoperators.memory import zeros
 from pyoperators.rules import rule_manager
@@ -23,8 +24,8 @@ from pyoperators.utils import (
     product)
 from pyoperators.utils.mpi import MPI, distribute_slice
 from pyoperators.utils.testing import (
-    assert_eq, assert_is, assert_is_not, assert_is_none, assert_not_in,
-    assert_is_instance, assert_raises, assert_is_type, assert_same, skiptest)
+    assert_eq, assert_is, assert_is_none, assert_is_instance, assert_raises,
+    assert_is_type, assert_same, skiptest)
 from scipy.sparse import csc_matrix
 from .common import OPS, ALL_OPS, DTYPES, HomothetyOutplaceOperator
 
@@ -124,10 +125,10 @@ def test_flags():
                 v = np.arange(10.)
                 o = op(v)
             except:
-                print 'Cannot test: ' + op.__name__
+                print('Cannot test: ' + op.__name__)
                 return
         if type(o) is not op:
-            print 'Cannot test: ' + op.__name__
+            print('Cannot test: ' + op.__name__)
             return
         if o.flags.idempotent:
             assert_is(o, o(o))
@@ -875,9 +876,9 @@ def test_propagation_attribute1():
         yield func1, i
 
     def func2(i_):
-        print
-        print 'op'
-        print '=='
+        print()
+        print('op')
+        print('==')
         op = AddAttribute()
         i = i_.copy()
         assert op(i, i).newattr_direct
@@ -885,9 +886,9 @@ def test_propagation_attribute1():
         assert op.T(i, i).newattr_transpose
 
         pool.clear()
-        print
-        print 'op2 * op'
-        print '======='
+        print()
+        print('op2 * op')
+        print('=======')
         op = AddAttribute2() * AddAttribute()
         i = i_.copy()
         assert not op(i, i).newattr_direct
@@ -895,9 +896,9 @@ def test_propagation_attribute1():
         assert op.T(i, i).newattr_transpose
 
         pool.clear()
-        print
-        print 'op3 * op'
-        print '======='
+        print()
+        print('op3 * op')
+        print('=======')
         op = AddAttribute3() * AddAttribute()
         i = i_.copy()
         o = op(i, i)
@@ -2185,268 +2186,6 @@ def test_copy():
     assert_equal(x, x_)
 
 
-#=============================
-# Test broadcasting operators
-#=============================
-
-def test_diagonal1():
-    data = (0., 1., [0, 0], [1, 1], 2, [2, 2], [0, 1], [-1, -1], [-1, 1],
-            [2, 1])
-    expected = (ZeroOperator, IdentityOperator, ZeroOperator, IdentityOperator,
-                HomothetyOperator, HomothetyOperator, MaskOperator,
-                HomothetyOperator, DiagonalOperator, DiagonalOperator)
-
-    def func(d, e):
-        op = DiagonalOperator(d)
-        if all(_ in (-1, 1) for _ in op.data.flat):
-            assert op.flags.involutary
-        assert_is(op.__class__, e)
-    for d, e in zip(data, expected):
-        yield func, d, e
-
-
-def test_diagonal2():
-    ops = (DiagonalOperator([1., 2], broadcast='rightward'),
-           DiagonalOperator([[2., 3, 4], [5, 6, 7]], broadcast='rightward'),
-           DiagonalOperator([1., 2, 3, 4, 5], broadcast='leftward'),
-           DiagonalOperator(np.arange(20).reshape(4, 5), broadcast='leftward'),
-           DiagonalOperator(np.arange(120.).reshape(2, 3, 4, 5)),
-           HomothetyOperator(7.),
-           IdentityOperator())
-
-    def func(op, d1, d2):
-        d = op(d1, d2)
-        if type(d1) is DiagonalOperator:
-            assert_is(type(d), DiagonalOperator)
-        elif type(d1) is HomothetyOperator:
-            assert_is(type(d), HomothetyOperator)
-        elif op is np.multiply:
-            assert_is(type(d), IdentityOperator)
-        else:
-            assert_is(type(d), HomothetyOperator)
-
-        data = op(d1.data.T, d2.data.T).T if 'rightward' in (d1.broadcast,
-               d2.broadcast) else op(d1.data, d2.data)
-        assert_eq(d.data, data)
-    for op in (np.add, np.multiply):
-        for i in range(7):
-            d1 = ops[i]
-            for j in range(i, 7):
-                d2 = ops[j]
-                if set((d1.broadcast, d2.broadcast)) == \
-                   set(('leftward', 'rightward')):
-                    continue
-                yield func, op, d1, d2
-
-
-def test_masking():
-    mask = MaskOperator(0)
-    assert isinstance(mask, IdentityOperator)
-    mask = MaskOperator(0, shapein=(32, 32), dtype=np.float32)
-    assert isinstance(mask, IdentityOperator)
-    assert mask.shapein == (32, 32)
-    assert mask.dtype == np.float32
-
-    mask = MaskOperator(1)
-    assert isinstance(mask, ZeroOperator)
-    mask = MaskOperator(1, shapein=(32, 32), dtype=np.float32)
-    assert isinstance(mask, ZeroOperator)
-    assert mask.shapein == (32, 32)
-    assert mask.dtype == np.float32
-
-    b = np.array([3., 4., 1., 0., 3., 2.])
-    c = np.array([3., 4., 0., 0., 3., 0.])
-    mask = MaskOperator(np.array([0, 0., 1., 1., 0., 1], dtype=np.int8))
-    assert np.all(mask(b) == c)
-    mask = DiagonalOperator(np.array([1, 1., 0., 0., 1., 0]))
-    assert np.all(mask(b) == c)
-    mask = MaskOperator(np.array([False, False, True, True, False, True]))
-    assert np.all(mask(b) == c)
-
-    b = np.array([[3., 4.], [1., 0.], [3., 2.]])
-    c = np.array([[3., 4.], [0., 0.], [3., 0.]])
-    mask = MaskOperator(np.array([[0, 0.], [1., 1.], [0., 1.]], dtype='int8'))
-    assert np.all(mask(b) == c)
-    mask = DiagonalOperator(np.array([[1, 1.], [0., 0.], [1., 0.]]))
-    assert np.all(mask(b) == c)
-    mask = MaskOperator(np.array([[False, False],
-                                  [True, True],
-                                  [False, True]]))
-    assert np.all(mask(b) == c)
-
-    b = np.array([[[3, 4.], [1., 0.]], [[3., 2], [-1, 9]]])
-    c = np.array([[[3, 4.], [0., 0.]], [[3., 0], [0, 0]]])
-    mask = MaskOperator(np.array([[[0, 0.], [1., 1.]],
-                                  [[0., 1], [1, 1]]], int))
-    assert np.all(mask(b) == c)
-
-    mask = DiagonalOperator(np.array([[[1, 1], [0., 0]], [[1, 0], [0, 0]]]))
-    assert np.all(mask(b) == c)
-    mask = MaskOperator(np.array([[[False, False], [True, True]],
-                                  [[False, True], [True, True]]]))
-    assert np.all(mask(b) == c)
-
-    c = mask(b, b)
-    assert id(b) == id(c)
-
-
-def test_masking2():
-    m = MaskOperator([True, False, True])
-    assert_eq(m * m,  m)
-
-
-def test_homothety_operator():
-    s = HomothetyOperator(1)
-    assert s.C is s.T is s.H is s.I is s
-
-    s = HomothetyOperator(-1)
-    assert s.C is s.T is s.H is s.I is s
-
-    s = HomothetyOperator(2.)
-    assert s.C is s.T is s.H is s
-    assert_is_not(s.I, s)
-
-    def func(o):
-        assert_is_instance(o, HomothetyOperator)
-    for o in (s.I, s.I.C, s.I.T, s.I.H, s.I.I):
-        yield func, o
-
-    s = HomothetyOperator(complex(1, 1))
-    assert_is(s.T, s)
-    assert_is(s.H, s.C)
-    assert_not_in(s.I, (s, s.C))
-    assert_not_in(s.I.C, (s, s.C))
-    assert_is_instance(s.C, HomothetyOperator)
-    for o in (s.I, s.I.C, s.I.T, s.I.H, s.I.I):
-        yield func, o
-
-
-def test_homothety_reduction1():
-    models = 1.*I+I, -I, (-2) * I, -(2*I), 1.*I-I, 1.*I-2*I
-    results = [6, -3, -6, -6, 0, -3]
-
-    def func(model, result, i):
-        o = model(i)
-        assert_eq(o, result, str((model, i)))
-        assert_eq(o.dtype, int, str((model, i)))
-    for model, result in zip(models, results):
-        for i in (np.array(3), [3], (3,), np.int(3), 3):
-            yield func, model, result, i
-
-
-def test_homothety_reduction2():
-    model = -I
-    iops = (operator.iadd, operator.isub, operator.imul, operator.iadd,
-            operator.imul)
-    imodels = 2*I, 2*I, 2*I, O, O
-    results = [3, -3, -6, -6, 0]
-
-    def func(imodel, result, i):
-        assert_eq(model(i), result)
-    for iop, imodel, result in zip(iops, imodels, results):
-        model = iop(model, imodel)
-        for i in (np.array(3), [3], (3,), np.int(3), 3):
-            yield func, imodel, result, i
-
-
-def test_homothety_reduction3():
-    @flags.linear
-    class Op(Operator):
-        pass
-
-    def func(opout, opin, idin):
-        if opin is not None and idin is not None and opin != idin:
-            return
-        p = Op(shapeout=opout, shapein=opin) * IdentityOperator(shapein=idin)
-
-        if idin is None:
-            idin = opin
-        assert_is_instance(p, Op)
-        assert_eq(p.shapein, idin)
-        assert_eq(p.shapeout, opout)
-    for opout in (None, (100,)):
-        for opin in (None, (100,)):
-            for idin in (None, (100,)):
-                yield func, opout, opin, idin
-
-
-def test_constant_reduction1():
-    c = 1, np.array([1, 2]), np.array([2, 3, 4])
-    t = 'scalar', 'rightward', 'leftward'
-
-    def func(c1, t1, c2, t2):
-        op2 = ConstantOperator(c2, broadcast=t2)
-        op = op1 + op2
-        if set((op1.broadcast, op2.broadcast)) != \
-           set(('rightward', 'leftward')):
-            assert_is_instance(op, ConstantOperator)
-        v = np.zeros((2, 3))
-        op(np.nan, v)
-        z = np.zeros((2, 3))
-        if t1 == 'rightward':
-            z.T[...] += c1.T
-        else:
-            z[...] += c1
-        if t2 == 'rightward':
-            z.T[...] += c2.T
-        else:
-            z[...] += c2
-        assert_eq(v, z)
-    for c1, t1 in zip(c, t):
-        op1 = ConstantOperator(c1, broadcast=t1)
-        for c2, t2 in zip(c, t):
-            yield func, c1, t1, c2, t2
-
-
-def test_constant_reduction2():
-    H = HomothetyOperator
-    C = CompositionOperator
-    D = DiagonalOperator
-    cs = (ConstantOperator(3),
-          ConstantOperator([1, 2, 3], broadcast='leftward'),
-          ConstantOperator(np.ones((2, 3))))
-    os = (I, H(2, shapein=(2, 3)) * Operator(direct=np.square, shapein=(2, 3),
-                                             flags='linear,square'), H(5))
-    results = (((H, 3), (C, (H, 6)), (H, 15)),
-               ((D, [1, 2, 3]), (C, (D, [2, 4, 6])), (D, [5, 10, 15])),
-               ((IdentityOperator, 1), (C, (H, 2)), (H, 5)))
-    v = np.arange(6).reshape((2, 3))
-
-    def func(c, o, r):
-        op = MultiplicationOperator([c, o])
-        assert_eq(op(v), c.data*o(v))
-        assert_is(type(op), r[0])
-        if type(op) is CompositionOperator:
-            op = op.operands[0]
-            r = r[1]
-            assert_is(type(op), r[0])
-        assert_eq, op.data, r[1]
-    for c, rs in zip(cs, results):
-        for o, r in zip(os, rs):
-            yield func, c, o, r
-
-
-def _test_constant_reduction3():
-    @flags.square
-    class Op(Operator):
-        def direct(self, input, output):
-            output[...] = input + np.arange(input.size).reshape(input.shape)
-
-    os = (Op(shapein=()), Op(shapein=4), Op(shapein=(2, 3, 4)))
-    cs = (ConstantOperator(2), ConstantOperator([2], broadcast='leftward'),
-          ConstantOperator(2*np.arange(8).reshape((2, 1, 4)),
-                           broadcast='leftward'))
-    v = 10000000
-
-    def func(o, c):
-        op = o * c
-        y_tmp = np.empty(o.shapein, int)
-        c(v, y_tmp)
-        assert_eq(op(v), o(y_tmp))
-    for o, c in zip(os, cs):
-        yield func, o, c
-
-
 #========================
 # Test ReductionOperator
 #========================
